@@ -19,6 +19,7 @@ package bincommon
 // Do not add any external dependencies we want to keep fortio minimal.
 
 import (
+	"context"
 	"flag"
 	"fmt"
 	"io"
@@ -31,6 +32,7 @@ import (
 	"fortio.org/fortio/fhttp"
 	"fortio.org/fortio/fnet"
 	"fortio.org/fortio/log"
+	"fortio.org/fortio/periodic"
 	"fortio.org/fortio/version"
 )
 
@@ -47,6 +49,10 @@ func (f *headersFlagList) Set(value string) error {
 
 // -- end of functions for -H support
 
+// FortioHook is used in cli and rapi to customize the run and introduce for instance clienttrace
+// and otel access logger.
+type FortioHook func(*fhttp.HTTPOptions, *periodic.RunnerOptions)
+
 // FlagsUsage prints end of the usage() (flags part + error message).
 func FlagsUsage(w io.Writer, msgs ...interface{}) {
 	_, _ = fmt.Fprintf(w, "flags are:\n")
@@ -65,6 +71,7 @@ var (
 	httpReqTimeoutFlag  = flag.Duration("timeout", fhttp.HTTPReqTimeOutDefaultValue, "Connection and read timeout value (for http)")
 	stdClientFlag       = flag.Bool("stdclient", false, "Use the slower net/http standard client (slower but supports h2)")
 	http10Flag          = flag.Bool("http1.0", false, "Use http1.0 (instead of http 1.1)")
+	h2Flag              = flag.Bool("h2", false, "Attempt to use http2.0 / h2 (instead of http 1.1) with stdclient and TLS")
 	httpsInsecureFlag   = flag.Bool("k", false, "Do not verify certs in https/tls/grpc connections")
 	httpsInsecureFlagL  = flag.Bool("https-insecure", false, "Long form of the -k flag")
 	resolve             = flag.String("resolve", "", "Resolve host name to this `IP`")
@@ -119,7 +126,8 @@ var (
 
 // SharedMain is the common part of main from fortio_main and fcurl.
 func SharedMain(usage func(io.Writer, ...interface{})) {
-	flag.Var(&headersFlags, "H", "Additional `header`(s)")
+	flag.Var(&headersFlags, "H",
+		"Additional http header(s) or grpc metadata. Multiple `key:value` pairs can be passed using multiple -H.")
 	flag.IntVar(&fhttp.BufferSizeKb, "httpbufferkb", fhttp.BufferSizeKb,
 		"Size of the buffer (max data size) for the optimized http client in `kbytes`")
 	flag.BoolVar(&fhttp.CheckConnectionClosedHeader, "httpccch", fhttp.CheckConnectionClosedHeader,
@@ -154,7 +162,7 @@ func FetchURL(o *fhttp.HTTPOptions) {
 	if client == nil || reflect.ValueOf(client).IsNil() {
 		os.Exit(1) // error logged already
 	}
-	code, data, header := client.Fetch()
+	code, data, header := client.Fetch(context.Background())
 	log.LogVf("Fetch result code %d, data len %d, headerlen %d", code, len(data), header)
 	if *curlHeadersStdout {
 		os.Stdout.Write(data)
@@ -197,6 +205,7 @@ func SharedHTTPOptions() *fhttp.HTTPOptions {
 	url := strings.TrimLeft(flag.Arg(0), " \t\r\n")
 	httpOpts.URL = url
 	httpOpts.HTTP10 = *http10Flag
+	httpOpts.H2 = *h2Flag
 	httpOpts.DisableFastClient = *stdClientFlag
 	httpOpts.DisableKeepAlive = !*keepAliveFlag
 	httpOpts.AllowHalfClose = *halfCloseFlag
