@@ -15,6 +15,7 @@
 package fhttp
 
 import (
+	"context"
 	"fmt"
 	"net/http"
 	"os"
@@ -56,9 +57,9 @@ type HTTPRunnerResults struct {
 
 // Run tests http request fetching. Main call being run at the target QPS.
 // To be set as the Function in RunnerOptions.
-func (httpstate *HTTPRunnerResults) Run(t int) (bool, string) {
+func (httpstate *HTTPRunnerResults) Run(ctx context.Context, t periodic.ThreadID) (bool, string) {
 	log.Debugf("Calling in %d", t)
-	code, body, headerSize := httpstate.client.Fetch()
+	code, body, headerSize := httpstate.client.Fetch(ctx)
 	size := len(body)
 	log.Debugf("Got in %3d hsz %d sz %d - will abort on %d", code, headerSize, size, httpstate.AbortOn)
 	httpstate.RetCodes[code]++
@@ -124,6 +125,7 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 	httpstate := make([]HTTPRunnerResults, numThreads)
 	// First build all the clients sequentially. This ensures we do not have data races when
 	// constructing requests.
+	ctx := context.Background()
 	for i := 0; i < numThreads; i++ {
 		r.Options().Runners[i] = &httpstate[i]
 		// Temp mutate the option so each client gets a logging id
@@ -136,7 +138,7 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 			return nil, err
 		}
 		if o.SequentialWarmup && o.Exactly <= 0 {
-			code, data, headerSize := httpstate[i].client.Fetch()
+			code, data, headerSize := httpstate[i].client.Fetch(ctx)
 			if !o.AllowInitialErrors && !codeIsOK(code) {
 				return nil, fmt.Errorf("error %d for %s: %q", code, o.URL, string(data))
 			}
@@ -156,7 +158,7 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 		for i := 0; i < numThreads; i++ {
 			i := i
 			warmup.Go(func() error {
-				code, data, headerSize := httpstate[i].client.Fetch()
+				code, data, headerSize := httpstate[i].client.Fetch(ctx)
 				if !o.AllowInitialErrors && !codeIsOK(code) {
 					return fmt.Errorf("error %d for %s: %q", code, o.URL, string(data))
 				}
@@ -208,8 +210,9 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 		occurrence, connStats := httpstate[i].client.GetIPAddress()
 		currentSocketUsed := connStats.Count
 		httpstate[i].client.Close()
-		fmt.Fprintf(out, "[%d] %3d socket used, resolved to %s ", i, currentSocketUsed, occurrence.PrintAndAggregate(total.IPCountMap))
-		connStats.Counter.Print(out, "connection timing")
+		// next 2 in 1 (long) line:
+		fmt.Fprintf(out, "[%d] %3d socket used, resolved to %s", i, currentSocketUsed, occurrence.AggregateAndToString(total.IPCountMap))
+		connStats.Counter.Print(out, ", connection timing")
 		total.SocketCount += currentSocketUsed
 		total.Sockets = append(total.Sockets, currentSocketUsed)
 		// Q: is there some copying each time stats[i] is used?
@@ -245,7 +248,7 @@ func RunHTTPTest(o *HTTPRunnerOptions) (*HTTPRunnerResults, error) {
 	sort.Ints(keys)
 	totalCount := float64(total.DurationHistogram.Count)
 	_, _ = fmt.Fprintf(out, "Sockets used: %d (for perfect keepalive, would be %d)\n", total.SocketCount, r.Options().NumThreads)
-	_, _ = fmt.Fprintf(out, "Uniform: %t, Jitter: %t\n", total.Uniform, total.Jitter)
+	_, _ = fmt.Fprintf(out, "Uniform: %t, Jitter: %t, Catchup allowed: %t\n", total.Uniform, total.Jitter, !total.NoCatchUp)
 	_, _ = fmt.Fprintf(out, "IP addresses distribution:\n")
 	for _, v := range ipList {
 		_, _ = fmt.Fprintf(out, "%s: %d\n", v, total.IPCountMap[v])
