@@ -88,6 +88,8 @@ var (
 	dataDir string
 	// Default percentiles when not otherwise specified.
 	DefaultPercentileList []float64
+	// Hook to install OTEL or other options.
+	hook bincommon.FortioHook
 )
 
 // AsyncReply is returned when async=on is passed.
@@ -181,6 +183,7 @@ func FormValue(r *http.Request, json map[string]interface{}, key string) string
 }
 
 // RESTRunHandler is api version of UI submit handler.
+// TODO: refactor common option/args/flag parsing between uihandler.go and this.
 func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	fhttp.LogRequest(r, "REST Run Api call")
 	w.Header().Set("Content-Type", "application/json")
@@ -221,6 +224,7 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	uniform := (FormValue(r, jd, "uniform") == "on")
 	nocatchup := (FormValue(r, jd, "nocatchup") == "on")
 	stdClient := (FormValue(r, jd, "stdclient") == "on")
+	h2 := (FormValue(r, jd, "h2") == "on")
 	sequentialWarmup := (FormValue(r, jd, "sequential-warmup") == "on")
 	httpsInsecure := (FormValue(r, jd, "https-insecure") == "on")
 	resolve := FormValue(r, jd, "resolve")
@@ -264,12 +268,14 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	ro.RunID = runid
 	log.Infof("New run id %d", runid)
 	httpopts := &fhttp.HTTPOptions{}
-	httpopts.HTTPReqTimeOut = timeout // to be normalized in init 0 replaced by default value
-	httpopts = httpopts.Init(url)
+	httpopts.HTTPReqTimeOut = timeout // to be normalized in init, 0 is replaced by default value (for all runners)
+	// We don't call Init because this could be a tcp:// or udp:// url. Was httpopts = httpopts.Init(url) - fixes #651
+	httpopts.URL = url
 	httpopts.DisableFastClient = stdClient
 	httpopts.SequentialWarmup = sequentialWarmup
 	httpopts.Insecure = httpsInsecure
 	httpopts.Resolve = resolve
+	httpopts.H2 = h2
 	// Set the connection reuse range.
 	err = bincommon.ConnectionReuseRange.
 		WithValidator(bincommon.ConnectionReuseRangeValidator(httpopts)).
@@ -338,6 +344,9 @@ func Run(w http.ResponseWriter, r *http.Request, jd map[string]interface{},
 	var res periodic.HasRunnerResult
 	var err error
 	var aborter *periodic.Aborter
+	if hook != nil {
+		hook(httpopts, ro)
+	}
 	if runner == ModeGRPC { //nolint:nestif
 		grpcSecure := (FormValue(r, jd, "grpc-secure") == "on")
 		grpcPing := (FormValue(r, jd, "ping") == "on")
@@ -347,6 +356,7 @@ func Run(w http.ResponseWriter, r *http.Request, jd map[string]interface{},
 			Destination:   url,
 			UsePing:       grpcPing,
 			Delay:         grpcPingDelay,
+			Service:       FormValue(r, jd, "healthservice"),
 		}
 		o.TLSOptions = httpopts.TLSOptions
 		if grpcSecure {
@@ -523,7 +533,8 @@ func RemoveRun(id int64) {
 
 // AddHandlers adds the REST Api handlers for run, status and stop.
 // uiPath must end with a /.
-func AddHandlers(mux *http.ServeMux, baseurl, uiPath, datadir string) {
+func AddHandlers(ahook bincommon.FortioHook, mux *http.ServeMux, baseurl, uiPath, datadir string) {
+	hook = ahook
 	AddDataHandler(mux, baseurl, uiPath, datadir)
 	restRunPath := uiPath + RestRunURI
 	mux.HandleFunc(restRunPath, RESTRunHandler)
